from __future__ import annotations

from pathlib import Path

from bulot import Shell


README = Path(__file__).parent.parent / "README.md"


def extract_codes(language: str = "python") -> list[str]:
    blocks = []
    with open(README) as f:
        block: list[str] = []
        is_inside_block = False

        for line in f.read().split("\n"):
            if line == f"```{language}":
                is_inside_block = True
            elif is_inside_block and line == "```":
                blocks.append("\n".join(block))
                block.clear()
                is_inside_block = False
            elif is_inside_block:
                block.append(line)

    return blocks


class TestReadmeCodeBlocks:
    blocks_python = extract_codes()

    def test_python(self):
        assert len(self.blocks_python) == 4
        for block in self.blocks_python:
            sh = Shell()
            sh.cd(Path(__file__).parent.parent)
            exec(block, globals())



import pytest
from unittest import mock


@pytest.fixture(scope="function")
def file_mocker():
    # Read a mocked /etc/release file
    mocked_file = mock.mock_open()
    # with mock.patch("__main__.open", mocked_file), mock.patch(
    #     "pathlib.Path.exists", lambda: True
    # ):
    with mock.patch("builtins.open", mocked_file):
        yield mocked_file



import subprocess as sp
from pathlib import Path
from unittest.mock import Mock

import pytest

from bulot import Shell, Command
from .constants import ECHO_CMD, ECHO_STDOUT, FILE_NAME


class TestCommand:
    command = Shell()(ECHO_CMD)

    def test_command(self):
        command = self.command
        assert isinstance(command, Command)
        assert command.value == ECHO_CMD
        assert command.cwd == Path(".")
        assert command.start < command.end
        assert command.return_code == 0

        result = command.result
        assert isinstance(result, sp.CompletedProcess)
        assert result.args == ["echo", "Plop"]
        assert result.returncode == 0
        assert result.stdout == f"{ECHO_STDOUT}\n".encode("utf-8")
        assert result.stderr == "".encode("utf-8")

    @pytest.mark.usefixtures("file_mocker")
    def test__gt__(self, file_mocker: Mock):
        self.command > FILE_NAME
        file_mocker.assert_called_once_with(Path(FILE_NAME), "w")
        file_mocker().write.assert_called_once_with(ECHO_STDOUT)

    @pytest.mark.usefixtures("file_mocker")
    def test__rshift__(self, file_mocker: Mock):
        self.command >> FILE_NAME
        file_mocker.assert_called_once_with(Path(FILE_NAME), "a+")
        file_mocker().write.assert_called_once_with(ECHO_STDOUT)



import subprocess as sp

import pytest

from bulot import Shell, Command, Pipe
from .constants import (
    ECHO_CMD,
    ECHO_STDOUT,
    ECHO_STR,
    WRONG_CMD,
    WRONG_FILE,
    PATH_TESTS,
    PATH_DOES_NOT_EXIST,
)


class TestShell:
    sh = Shell()

    def test_run(self):
        command = self.sh.run(ECHO_CMD)
        assert isinstance(command, Command)
        assert command.stdout == ECHO_STDOUT

    def test_ls_absolute(self):
        self.sh.cd(PATH_TESTS.parent)
        assert self.sh.cwd() == PATH_TESTS.parent
        self.sh.cd(PATH_TESTS)
        assert self.sh.cwd() == PATH_TESTS

    def test_ls_relative(self):
        directory_name = PATH_TESTS.parts[-1]
        self.sh.cd(PATH_TESTS.parent)
        assert self.sh.cwd() == PATH_TESTS.parent
        self.sh.cd(directory_name)
        assert self.sh.cwd() == PATH_TESTS

    def test_ls_wrong_path(self):
        self.sh.cd(PATH_TESTS.parent)
        assert self.sh.cwd() == PATH_TESTS.parent
        with pytest.raises(sp.CalledProcessError):
            self.sh.cd(PATH_DOES_NOT_EXIST)
        assert self.sh.cwd() == PATH_TESTS.parent

    def test_error(self):
        command = self.sh(WRONG_CMD)
        assert isinstance(command, Command)
        assert command.return_code == 2
        assert command.stdout == ""
        assert command.stderr.__contains__(WRONG_FILE)

    def test_error_check(self):
        with pytest.raises(sp.CalledProcessError):
            self.sh(WRONG_CMD, check=True)

        command = self.sh(WRONG_CMD)
        assert command.return_code != 0

        self.sh.config.check = True
        with pytest.raises(sp.CalledProcessError):
            self.sh(WRONG_CMD)
        with pytest.raises(sp.CalledProcessError):
            self.sh(WRONG_CMD)

        command = self.sh(WRONG_CMD, check=False)
        assert command.return_code != 0

    def test__call__(self):
        command = self.sh(ECHO_CMD)
        assert isinstance(command, Command)
        assert command.stdout == ECHO_STDOUT

    def test__or__(self):
        pipe = self.sh | ECHO_CMD
        assert isinstance(pipe, Pipe)
        assert pipe.stdout == ECHO_STDOUT
        assert pipe.sh is self.sh

    def test__lshift__(self):
        pipe = self.sh << ECHO_STR
        assert isinstance(pipe, Pipe)
        assert pipe.stdout == ECHO_STR
        assert pipe.sh is self.sh



import sys


def cmp(a: int, b: int) -> int:
    return (a > b) - (a < b)


class PythonVersion:
    version = sys.version_info[:3]

    def compare(self, expected_str: str) -> int:
        expected: list[int] = list(map(int, expected_str.split(".")))

        for exp, ver in zip(expected, self.version):
            comparaison = cmp(ver, exp)
            if comparaison != 0:
                return comparaison

        return 0

    def __eq__(self, expected: object) -> bool:
        if not isinstance(expected, str):
            raise NotImplementedError(f"Can't handle type '{type(expected)}' only str")
        return self.compare(expected) == 0

    def __lt__(self, expected: str) -> bool:
        return self.compare(expected) < 0

    def __le__(self, expected: str) -> bool:
        return self.compare(expected) <= 0

    def __gt__(self, expected: str) -> bool:
        return self.compare(expected) > 0

    def __ge__(self, expected: str) -> bool:
        return self.compare(expected) >= 0


PY = PythonVersion()
